/*******************************************************************************
 *                                                                             *
 *  falcON_F.c                                                                 *
 *                                                                             *
 *  written by Walter Dehnen, 2000/2001                                        *
 *  e-mail:   dehnen@mpia-hd.mpg.de                                            *
 *  address:  Max-Planck Institut fr Astronomie,                              *
 *            Knigstuhl 17, D-69117 Heidelberg                                *
 *            Germany                                                          *
 *                                                                             *
 ******************************************************************************/
#include <falcON_C.h>

#if defined(SINGLE_DOUBLE) || defined(DOUBLE_DOUBLE)
#define INPUT_TYPE double                     /*   define input type = double */
#else                                         /*                              */
#define INPUT_TYPE float                      /*   define input type = float  */
#endif                                        /*                              */

#if NDIM==3
#define POSIS   const INPUT_TYPE*X, const INPUT_TYPE*Y, const INPUT_TYPE*Z
#define POSS    X,Y,Z
#define VELIS   const INPUT_TYPE*VX, const INPUT_TYPE*VY, const INPUT_TYPE*VZ
#define VELS    VX,VY,VZ
#define ACCIS   INPUT_TYPE*AX, INPUT_TYPE*AY, INPUT_TYPE*AZ
#define ACCS    AX,AY,AZ
#define ZEROS   0,0,0
#else
#define POSIS   const INPUT_TYPE*X, const INPUT_TYPE*Y
#define POSS    X,Y
#define VELIS   const INPUT_TYPE*VX, const INPUT_TYPE*VY
#define VELS    VX,VY
#define ACCIS   INPUT_TYPE*AX, INPUT_TYPE*AY
#define ACCS    AX,AY
#define ZEROS   0,0
#endif

/******************************************************************************/
void falcon_init_ (int *F, INPUT_TYPE*M, POSIS, INPUT_TYPE*E, ACCIS,
		  INPUT_TYPE*PH, INPUT_TYPE*RH, int*N, INPUT_TYPE*EPS,
		  INPUT_TYPE*TH, int*K, int*DMAX)
{ falcON_initialize(F,M,POSS,E,ACCS,PH,RH,*N,*EPS,*TH,*K,*DMAX); }
void falcon_init__(int *F, INPUT_TYPE*M, POSIS, INPUT_TYPE*E, ACCIS,
		  INPUT_TYPE*PH, INPUT_TYPE*RH, int*N, INPUT_TYPE*EPS,
		  INPUT_TYPE*TH, int*K, int*DMAX)
{ falcON_initialize(F,M,POSS,E,ACCS,PH,RH,*N,*EPS,*TH,*K,*DMAX); }
/******************************************************************************/
void falcon_resetsoftening_ (INPUT_TYPE*EPS, int*K)
                                             { falcON_resetsoftening(*EPS,*K); }
void falcon_resetsoftening__(INPUT_TYPE*EPS, int*K)
                                             { falcON_resetsoftening(*EPS,*K); }
/******************************************************************************/
void falcon_resetopening_ (INPUT_TYPE*TH)     { falcON_resetopening(*TH); }
void falcon_resetopening__(INPUT_TYPE*TH)     { falcON_resetopening(*TH); }
/******************************************************************************/
void falcon_clearup_ ()                       { falcON_clearup(); }
void falcon_clearup__()                       { falcON_clearup(); }
/******************************************************************************/
void falcon_grow_ (int*nx)                    { falcON_grow(*nx); }
void falcon_grow__(int*nx)                    { falcON_grow(*nx); }
/******************************************************************************/
void falcon_grow_centered_ (int*nx,int X[NDIM]) { falcON_grow_centered(*nx,X); }
void falcon_grow_centered__(int*nx,int X[NDIM]) { falcON_grow_centered(*nx,X); }
/******************************************************************************/
void falcon_reuse_ ()                         { falcON_reuse(); }
void falcon_reuse__()                         { falcON_reuse(); }
/******************************************************************************/
void falcon_approx_grav_ ()                   { falcON_approx_grav(); }
void falcon_approx_grav__()                   { falcON_approx_grav(); }
/******************************************************************************/
void falcon_estimate_rho_ (int*nx)            { falcON_estimate_rho(*nx); }
void falcon_estimate_rho__(int*nx)            { falcON_estimate_rho(*nx); }
/******************************************************************************/
void falcon_estimate_n_ (int*nx)              { falcON_estimate_n(*nx); }
void falcon_estimate_n__(int*nx)              { falcON_estimate_n(*nx); }
/******************************************************************************/
void falcon_sticky_ (INPUT_TYPE*t,int*i1,int*i2,int*ni,int*na,
		     VELIS,INPUT_TYPE*s) {
  falcON_ialist(i1,i2,*ni,na,s,*t,VELS); }
void falcon_sticky__(INPUT_TYPE*t,int*i1,int*i2,int*ni,int*na,
		     VELIS,INPUT_TYPE*s) {
  falcON_ialist(i1,i2,*ni,na,s,*t,VELS); }
/******************************************************************************/
void falcon_sph_ (int*i1,int*i2,int*ni,int*na,INPUT_TYPE*s) {
  falcON_ialist(i1,i2,*ni,na,s,-1,ZEROS); }
void falcon_sph__(int*i1,int*i2,int*ni,int*na,INPUT_TYPE*s) {
  falcON_ialist(i1,i2,*ni,na,s,-1,ZEROS); }
/******************************************************************************/
float falcon_root_center_ (int*I)           { return falcON_root_center(*I); }
float falcon_root_center__(int*I)           { return falcON_root_center(*I); }
/******************************************************************************/
float falcon_root_radius_ ()                { return falcON_root_radius(); }
float falcon_root_radius__()                { return falcON_root_radius(); }
/******************************************************************************/
float falcon_current_theta_ (INPUT_TYPE*m)  { return falcON_current_theta(*m); }
float falcon_current_theta__(INPUT_TYPE*m)  { return falcON_current_theta(*m); }
/******************************************************************************/
float falcon_current_eps_ ()                { return falcON_current_eps(); }
float falcon_current_eps__()                { return falcON_current_eps(); }
/******************************************************************************/
int falcon_current_kernel_ ()               { return falcON_current_kernel(); }
int falcon_current_kernel__()               { return falcON_current_kernel(); }
/******************************************************************************/
int falcon_softening_ ()                    { return falcON_softening(); }
int falcon_softening__()                    { return falcON_softening(); }
/******************************************************************************/
int falcon_no_cells_ ()                     { return falcON_No_cells(); }
int falcon_no_cells__()                     { return falcON_No_cells(); }
/******************************************************************************/
int falcon_depth_ ()                        { return falcON_depth(); }
int falcon_depth__()                        { return falcON_depth(); }
/******************************************************************************/
void falcon_stats_ ()                       { falcON_stats(); }
void falcon_stats__()                       { falcON_stats(); }
/******************************************************************************/
#undef POSS
#undef VELS
#undef ACCS
#undef POSIS
#undef VELIS
#undef ACCIS
#undef ZEROS
/******************************************************************************/
#undef INPUT_TYPE                             /* not to be used elsewhere     */
/******************************************************************************
 ******************************************************************************/
#include <time.h>
#include <stdlib.h>
int    clock_()                { return clock(); }
int    clock__()               { return clock(); }
double drand48_()              { return drand48(); }  
double drand48__()             { return drand48(); }
double clocks_per_second_()    { return CLOCKS_PER_SEC; }
double clocks_per_second__()   { return CLOCKS_PER_SEC; }
void   srand48_(int*S)         { srand48(*S); }
void   srand48__(int*S)        { srand48(*S); }
/******************************************************************************
 ******************************************************************************/
